@@ -4,7 +4,7 @@ import math
 import urx
 from urx import ursecmon
 import math3d
-from hein_robots.robotics import Location, Twist, Wrench, Units
+from hein_robots.robotics import Location, Twist, Wrench, Units, Frame
 from hein_robots.robotiq.gripper import RobotiqGripper
 from hein_robots.base import robot_arms
 
@@ -133,18 +133,20 @@ class UR3Arm(robot_arms.RobotArm):
 
             time.sleep(0.01)
 
-    def move_to_location(self, location: Location,
+    def move_to_location(self, location: Location, frame: Optional[Frame] = None,
                          velocity: Optional[float] = None, acceleration: Optional[float] = None,
                          relative: bool = False, wait: bool = True, timeout: Optional[float] = None):
         vel = velocity if velocity is not None else self._default_velocity
         acc = acceleration if acceleration is not None else vel * 2
-        transform = self.robot.csys * math3d.Transform(location.convert_mm_to_m().matrix)
+        # use the reference frame (or an identity frame) to transform the relative location to a location in the base reference frame
+        relative_position = (frame or Location()) * location
+        transform = self.robot.csys * math3d.Transform(relative_position.convert_mm_to_m().matrix)
         distance = self.robot._get_lin_dist(transform.pose_vector)
 
         self.robot.movel(transform.pose_vector, acc=acc / 1000, vel=vel / 1000, relative=relative, wait=False)
 
         # don't wait if we aren't going to move
-        if wait and distance > 1e-4:
+        if wait and distance > 1e-3:
             self.wait(timeout, wait_for_running=True)
 
     def move_joints(self, joint_positions: List[float],
@@ -164,7 +166,23 @@ class UR3Arm(robot_arms.RobotArm):
             pass
 
         # don't wait if we aren't going to move
-        if wait and distance > 1e-4:
+        if wait and distance > 1e-3:
+            self.wait(timeout, wait_for_running=True)
+
+    def move_circular(self, midpoint: Location, end: Location, frame: Optional[Frame] = None,
+                      velocity: Optional[float] = None, acceleration: Optional[float] = None,
+                      wait: bool = True, timeout: Optional[float] = None):
+        vel = velocity if velocity is not None else self._default_velocity
+        acc = acceleration if acceleration is not None else vel * 2
+        # use the reference frame (or an identity frame) to transform the relative location to a location in the base reference frame
+        relative_midpoint = (frame or Location()) * midpoint
+        relative_end = (frame or Location()) * end
+        transform_midpoint = self.robot.csys * math3d.Transform(relative_midpoint.convert_mm_to_m().matrix)
+        transform_end = self.robot.csys * math3d.Transform(relative_end.convert_mm_to_m().matrix)
+
+        self.robot.movec(transform_midpoint.pose_vector, transform_end.pose_vector, acc=acc / 1000, vel=vel / 1000, wait=False)
+
+        if wait:
             self.wait(timeout, wait_for_running=True)
 
     def wait_for_gripper_stop(self, timeout: Optional[float] = None):
